@@ -5,7 +5,6 @@ import contextlib
 import inspect
 import traceback
 from itertools import islice
-from types import FunctionType
 from typing import TYPE_CHECKING
 
 import xcffib
@@ -71,8 +70,10 @@ _NET_WM_STATE_TOGGLE = 2
 
 
 def _geometry_getter(attr):
+    attr_name = "_" + attr
+
     def get_attr(self):
-        if getattr(self, "_" + attr) is None:
+        if getattr(self, attr_name) is None:
             g = self.window.get_geometry()
             # trigger the geometry setter on all these
             self.x = g.x
@@ -80,12 +81,14 @@ def _geometry_getter(attr):
             self.width = g.width
             self.height = g.height
             self.depth = g.depth
-        return getattr(self, "_" + attr)
+        return getattr(self, attr_name)
 
     return get_attr
 
 
 def _geometry_setter(attr):
+    attr_name = "_" + attr
+
     def f(self, value):
         if not isinstance(value, int):
             frame = inspect.currentframe()
@@ -93,7 +96,7 @@ def _geometry_setter(attr):
             logger.error("!!!! setting %s to a non-int %s; please report this!", attr, value)
             logger.error("".join(stack_trace[:-1]))
             value = int(value)
-        setattr(self, "_" + attr, value)
+        setattr(self, attr_name, value)
 
     return f
 
@@ -1239,7 +1242,7 @@ class _Window:
     def can_steal_focus(self, can_steal_focus: bool) -> None:
         self._can_steal_focus = can_steal_focus
 
-    def _do_focus(self):
+    def _do_focus(self) -> bool:
         """
         Focus the window if we can, and return whether or not it was successful.
         """
@@ -1540,6 +1543,7 @@ class Internal(_Window, base.Internal):
         return True
 
     def handle_EnterNotify(self, e):  # noqa: N802
+        self.qtile.hovered_window = self
         self.process_pointer_enter(e.event_x, e.event_y)
 
     def handle_LeaveNotify(self, e):  # noqa: N802
@@ -1744,6 +1748,8 @@ class Window(_Window, base.Window):
                 self._float_height = self.height
             self._float_state = FloatStates.NOT_FLOATING
             self.group.mark_floating(self, False)
+            if self.kept_above and self.qtile.config.floats_kept_above:
+                self.keep_above(enable=False)
             if tiled_stack:
                 self.window.configure(
                     stackmode=xcffib.xproto.StackMode.Above, sibling=tiled_stack[-1]
@@ -2039,6 +2045,7 @@ class Window(_Window, base.Window):
             return False
 
     def handle_EnterNotify(self, e):  # noqa: N802
+        self.qtile.hovered_window = self
         hook.fire("client_mouse_enter", self)
         if self.qtile.config.follow_mouse_focus is True:
             if self.group.current_window != self:
@@ -2138,59 +2145,11 @@ class Window(_Window, base.Window):
             self.window.set_property("_NET_WM_STATE", list(current_state))
         elif atoms["_NET_ACTIVE_WINDOW"] == opcode:
             source = data.data32[0]
-            if source == 2:  # XCB_EWMH_CLIENT_SOURCE_TYPE_NORMAL
+            if source == 2:  # Request from a pager should immediately focus the window
                 logger.debug("Focusing window by pager")
-                self.qtile.current_screen.set_group(self.group)
-                self.group.focus(self)
-                self.bring_to_front()
-            else:  # XCB_EWMH_CLIENT_SOURCE_TYPE_OTHER
-                focus_behavior = self.qtile.config.focus_on_window_activation
-                if (
-                    focus_behavior == "focus"
-                    or type(focus_behavior) is FunctionType
-                    and focus_behavior(self)
-                ):
-                    logger.debug("Focusing window")
-                    # Windows belonging to a scratchpad need to be toggled properly
-                    if isinstance(self.group, ScratchPad):
-                        for dropdown in self.group.dropdowns.values():
-                            if dropdown.window is self:
-                                dropdown.show()
-                                break
-                    else:
-                        self.qtile.current_screen.set_group(self.group)
-                        self.group.focus(self)
-                elif focus_behavior == "smart":
-                    if not self.group.screen:
-                        logger.debug(
-                            "Ignoring focus request (focus_on_window_activation='smart')"
-                        )
-                        return
-                    if self.group.screen == self.qtile.current_screen:
-                        logger.debug("Focusing window")
-                        # Windows belonging to a scratchpad need to be toggled properly
-                        if isinstance(self.group, ScratchPad):
-                            for dropdown in self.group.dropdowns.values():
-                                if dropdown.window is self:
-                                    dropdown.show()
-                                    break
-                        else:
-                            self.qtile.current_screen.set_group(self.group)
-                            self.group.focus(self)
-                    else:  # self.group.screen != self.qtile.current_screen:
-                        logger.debug("Setting urgent flag for window")
-                        self.urgent = True
-                        hook.fire("client_urgent_hint_changed", self)
-                elif focus_behavior == "urgent":
-                    logger.debug("Setting urgent flag for window")
-                    self.urgent = True
-                    hook.fire("client_urgent_hint_changed", self)
-                elif focus_behavior == "never":
-                    logger.debug("Ignoring focus request (focus_on_window_activation='never')")
-                else:
-                    logger.debug(
-                        "Invalid value for focus_on_window_activation: %s", focus_behavior
-                    )
+                self.activate()
+            else:  # Request from the application
+                self.activate_by_config()
         elif atoms["_NET_CLOSE_WINDOW"] == opcode:
             self.kill()
         elif atoms["WM_CHANGE_STATE"] == opcode:
