@@ -1,45 +1,15 @@
-# Copyright (c) 2008-2010 Aldo Cortesi
-# Copyright (c) 2011 Florian Mounier
-# Copyright (c) 2011 Kenji_Takahashi
-# Copyright (c) 2011 Paul Colomiets
-# Copyright (c) 2012 roger
-# Copyright (c) 2012 Craig Barnes
-# Copyright (c) 2012-2015 Tycho Andersen
-# Copyright (c) 2013 dequis
-# Copyright (c) 2013 David R. Andersen
-# Copyright (c) 2013 Tao Sauvage
-# Copyright (c) 2014-2015 Sean Vig
-# Copyright (c) 2014 Justin Bronder
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
-
 from __future__ import annotations
 
 import asyncio
 import copy
+import inspect
 import math
 import subprocess
 from typing import TYPE_CHECKING
 
-from libqtile import bar, configurable, confreader
+from libqtile import bar, configurable, confreader, hook
 from libqtile.command import interface
-from libqtile.command.base import CommandError, CommandObject, expose_command
+from libqtile.command.base import CommandObject, expose_command
 from libqtile.lazy import LazyCall
 from libqtile.log_utils import logger
 from libqtile.utils import ColorType, create_task
@@ -146,6 +116,7 @@ class _Widget(CommandObject, configurable.Configurable):
             {},
             "Dict of mouse button press callback functions. Accepts functions and ``lazy`` calls.",
         ),
+        ("hide_crash", False, "Don't display error in bar if widget crashes on startup."),
     ]
 
     def __init__(self, length, **config):
@@ -158,6 +129,13 @@ class _Widget(CommandObject, configurable.Configurable):
             self.name = config["name"]
 
         configurable.Configurable.__init__(self, **config)
+
+        # Add defaults for Mixins if inherited
+        if isinstance(self, PaddingMixin):
+            self.add_defaults(PaddingMixin.defaults)
+        if isinstance(self, MarginMixin):
+            self.add_defaults(MarginMixin.defaults)
+
         self.add_defaults(_Widget.defaults)
 
         if length in (bar.CALCULATED, bar.STRETCH):
@@ -177,7 +155,11 @@ class _Widget(CommandObject, configurable.Configurable):
     @property
     def length(self):
         if self.length_type == bar.CALCULATED:
-            return int(self.calculate_length())
+            try:
+                return int(self.calculate_length())
+            except Exception:
+                logger.exception(f"error when calculating widget {self.name} length")
+                return 0
         return self._length
 
     @length.setter
@@ -196,12 +178,6 @@ class _Widget(CommandObject, configurable.Configurable):
             return self.bar.height
         return self.length
 
-    @property
-    def offset(self):
-        if self.bar.horizontal:
-            return self.offsetx
-        return self.offsety
-
     def _test_orientation_compatibility(self, horizontal):
         if horizontal:
             if not self.orientations & ORIENTATION_HORIZONTAL:
@@ -236,6 +212,8 @@ class _Widget(CommandObject, configurable.Configurable):
 
             # Add these to our list of futures so they can be cancelled.
             self._futures.extend([timer, async_timer])
+            if hasattr(self, "force_update"):
+                hook.subscribe.resume(self.force_update)
 
     async def _config_async(self):
         """
@@ -244,7 +222,7 @@ class _Widget(CommandObject, configurable.Configurable):
 
         Widgets that need to use asyncio coroutines after this point may
         wish to initialise the relevant code (e.g. connections to dbus
-        using dbus_next) here.
+        using dbus_fast) here.
         """
 
     def finalize(self):
@@ -252,6 +230,7 @@ class _Widget(CommandObject, configurable.Configurable):
             future.cancel()
         if hasattr(self, "layout") and self.layout:
             self.layout.finalize()
+            self.layout = None
         self.drawer.finalize()
         self.finalized = True
 
@@ -268,7 +247,7 @@ class _Widget(CommandObject, configurable.Configurable):
         """Info for this object."""
         return dict(
             name=self.name,
-            offset=self.offset,
+            offset=self.offsetx if self.bar.horizontal else self.offsety,
             length=self.length,
             width=self.width,
             height=self.height,
@@ -296,15 +275,6 @@ class _Widget(CommandObject, configurable.Configurable):
     def button_release(self, x, y, button):
         pass
 
-    def get(self, q, name):
-        """
-        Utility function for quick retrieval of a widget by name.
-        """
-        w = q.widgets_map.get(name)
-        if not w:
-            raise CommandError(f"No such widget: {name}")
-        return w
-
     def _items(self, name: str) -> ItemT:
         if name == "bar":
             return True, []
@@ -318,6 +288,30 @@ class _Widget(CommandObject, configurable.Configurable):
         elif name == "screen":
             return self.bar.screen
 
+    def rotate_drawer_left(self):
+        # Left bar reads bottom to top
+        self.drawer.ctx.rotate(-90 * math.pi / 180.0)
+        self.drawer.ctx.translate(-self.length, 0)
+
+    def rotate_drawer_right(self):
+        # Right bar is top to bottom
+        self.drawer.ctx.translate(self.bar.width, 0)
+        self.drawer.ctx.rotate(90 * math.pi / 180.0)
+
+    def rotate_drawer(self):
+        if self.bar.horizontal:
+            return
+        if self.bar.screen.left is self.bar:
+            self.rotate_drawer_left()
+        elif self.bar.screen.right is self.bar:
+            self.rotate_drawer_right()
+
+    def draw_at_default_position(self):
+        """Default position to draw the widget in horizontal and vertical bars."""
+        self.drawer.draw(
+            offsetx=self.offsetx, offsety=self.offsety, width=self.width, height=self.height
+        )
+
     def draw(self):
         """
         Method that draws the widget. You may call this explicitly to
@@ -385,7 +379,7 @@ class _Widget(CommandObject, configurable.Configurable):
     def _wrapper(self, method, *method_args):
         self._remove_dead_timers()
         try:
-            if asyncio.iscoroutinefunction(method):
+            if inspect.iscoroutinefunction(method):
                 create_task(method(*method_args))
             elif asyncio.iscoroutine(method):
                 create_task(method)
@@ -423,7 +417,7 @@ class _Widget(CommandObject, configurable.Configurable):
     def add_mirror(self, widget: _Widget):
         if not self._mirrors:
             self._old_draw = self.draw
-            self.draw = self._draw_with_mirrors  # type: ignore
+            self.draw = self._draw_with_mirrors
 
         self._mirrors.add(widget)
         if not self.drawer.has_mirrors:
@@ -494,6 +488,15 @@ class _TextBox(_Widget):
             "When ``scroll=True`` the ``width`` parameter is a maximum width and, when text is shorter than this, the widget will resize. "
             "Setting ``scroll_fixed_width=True`` will force the widget to have a fixed width, regardless of the size of the text.",
         ),
+        ("rotate", True, "Rotate text in vertical bar."),
+        (
+            "direction",
+            "default",
+            "Override the text direction in vertical bar, has no effect on text in horizontal bar."
+            "default: text displayed based on vertical bar position (left/right)"
+            "ttb: text read from top to bottom, btt: text read from bottom to top."
+            "'default', 'ttb', 'btt'",
+        ),
     ]  # type: list[tuple[str, Any, str]]
 
     def __init__(self, text=" ", width=bar.CALCULATED, **config):
@@ -519,6 +522,7 @@ class _TextBox(_Widget):
         self._text = value
         if self.layout:
             self.layout.text = self.formatted_text
+
             if self.scroll:
                 self.check_width()
                 self.reset_scroll()
@@ -527,47 +531,19 @@ class _TextBox(_Widget):
     def formatted_text(self):
         return self.fmt.format(self._text)
 
-    @property
-    def foreground(self):
-        return self._foreground
-
-    @foreground.setter
-    def foreground(self, fg):
-        self._foreground = fg
-        if self.layout:
-            self.layout.colour = fg
-
-    @property
-    def font(self):
-        return self._font
-
-    @font.setter
-    def font(self, value):
-        self._font = value
-        if self.layout:
-            self.layout.font = value
-
-    @property
-    def fontshadow(self):
-        return self._fontshadow
-
-    @fontshadow.setter
-    def fontshadow(self, value):
-        self._fontshadow = value
-        if self.layout:
-            self.layout.font_shadow = value
-
-    @property
-    def actual_padding(self):
-        if self.padding is None:
-            return self.fontsize / 2
-        else:
-            return self.padding
-
     def _configure(self, qtile, bar):
         _Widget._configure(self, qtile, bar)
         if self.fontsize is None:
-            self.fontsize = self.bar.height - self.bar.height / 5
+            self.fontsize = self.bar.size - self.bar.size / 5
+        if self.padding is None:
+            self.padding = self.fontsize // 2
+        if self.direction not in ("default", "ttb", "btt"):
+            logger.warning(
+                "Invalid value set for direction: %s. Valid values are: 'default', 'ttb', 'btt'. "
+                "direction has been set to 'default'",
+                self.direction,
+            )
+            self.direction = "default"
         self.layout = self.drawer.textlayout(
             self.formatted_text,
             self.foreground,
@@ -577,8 +553,19 @@ class _TextBox(_Widget):
             markup=self.markup,
         )
         if not isinstance(self._scroll_width, int) and self.scroll:
-            logger.warning("%s: You must specify a width when enabling scrolling.", self.name)
-            self.scroll = False
+            if not self.bar.horizontal and not self.rotate:
+                self._scroll_width = self.bar.width
+            else:
+                logger.warning("%s: You must specify a width when enabling scrolling.", self.name)
+                self.scroll = False
+
+        # Setting the layout width will wrap text which increases layout's height,
+        # we only want this when bar is vertical and rotation is disabled
+        # to be able to display more of the text using multiple lines,
+        # only if scrolling is enabled the layout width will be overwritten
+        # because the widget's width is handle by scroll.
+        if not self.bar.horizontal and not self.rotate:
+            self.layout.width = self.bar.width
 
         if self.scroll:
             self.check_width()
@@ -588,13 +575,19 @@ class _TextBox(_Widget):
         Check whether the widget needs to have calculated or fixed width
         and whether the text should be scrolled.
         """
+        # Reset the layout width to let the layout calculate
+        # the width based on the length of the text.
+        self.layout.reset_width()
         if self.layout.width > self._scroll_width:
-            self.length_type = bar.STATIC
-            self.length = self._scroll_width
+            if self.bar.horizontal or self.rotate:
+                self.length_type = bar.STATIC
+                self.length = self._scroll_width
             self._is_scrolling = True
             self._should_scroll = True
         else:
-            if self.scroll_fixed_width:
+            if not self.bar.horizontal and not self.rotate:
+                self.layout.width = self.bar.width
+            elif self.scroll_fixed_width:
                 self.length_type = bar.STATIC
                 self.length = self._scroll_width
             else:
@@ -602,61 +595,53 @@ class _TextBox(_Widget):
             self._should_scroll = False
 
     def calculate_length(self):
-        if self.text:
-            if self.bar.horizontal:
-                return min(self.layout.width, self.bar.width) + self.actual_padding * 2
-            else:
-                return min(self.layout.width, self.bar.height) + self.actual_padding * 2
-        else:
+        if not self.text:
             return 0
+        if not self.bar.horizontal and not self.rotate:
+            return self.layout.height + self.padding * 2
+        else:
+            return min(self.layout.width, self.bar.length) + self.padding * 2
 
     def can_draw(self):
-        can_draw = (
-            self.layout is not None and not self.layout.finalized() and self.offsetx is not None
-        )  # if the bar hasn't placed us yet
-        return can_draw
+        return self.layout is not None
+
+    def rotate_drawer(self):
+        if self.bar.horizontal or not self.rotate:
+            return
+        # Execute the base method when direction is default
+        if self.direction == "default":
+            _Widget.rotate_drawer(self)
+        # Read bottom to top always with 'btt' direction
+        elif self.direction == "btt":
+            self.rotate_drawer_left()
+        # Read top to bottom always with 'ttb' direction
+        elif self.direction == "ttb":
+            self.rotate_drawer_right()
 
     def draw(self):
         if not self.can_draw():
             return
         self.drawer.clear(self.background or self.bar.background)
-
-        # size = self.bar.height if self.bar.horizontal else self.bar.width
         self.drawer.ctx.save()
-
-        if not self.bar.horizontal:
-            # Left bar reads bottom to top
-            if self.bar.screen.left is self.bar:
-                self.drawer.ctx.rotate(-90 * math.pi / 180.0)
-                self.drawer.ctx.translate(-self.length, 0)
-
-            # Right bar is top to bottom
-            else:
-                self.drawer.ctx.translate(self.bar.width, 0)
-                self.drawer.ctx.rotate(90 * math.pi / 180.0)
+        self.rotate_drawer()
 
         # If we're scrolling, we clip the context to the scroll width less the padding
         # Move the text layout position (and we only see the clipped portion)
         if self._should_scroll:
-            self.drawer.ctx.rectangle(
-                self.actual_padding,
-                0,
-                self._scroll_width - 2 * self.actual_padding,
-                self.bar.size,
-            )
+            height = self.bar.size if self.bar.horizontal or self.rotate else self.length
+            self.drawer.ctx.rectangle(0, 0, self._scroll_width, height)
             self.drawer.ctx.clip()
 
-        size = self.bar.height if self.bar.horizontal else self.bar.width
+        if not self.bar.horizontal and not self.rotate:
+            x, y = 0, self.padding
+        else:
+            x = self.padding if self.length_type != bar.STATIC else 0
+            y = (self.bar.size - self.layout.height) / 2 + 1
 
-        self.layout.draw(
-            (self.actual_padding or 0) - self._scroll_offset,
-            int(size / 2.0 - self.layout.height / 2.0) + 1,
-        )
+        self.layout.draw(x - self._scroll_offset, y)
         self.drawer.ctx.restore()
 
-        self.drawer.draw(
-            offsetx=self.offsetx, offsety=self.offsety, width=self.width, height=self.height
-        )
+        self.draw_at_default_position()
 
         # We only want to scroll if:
         # - User has asked us to scroll and the scroll width is smaller than the layout (should_scroll=True)
@@ -683,8 +668,7 @@ class _TextBox(_Widget):
         # - the final pixel is visible (scroll_clear = False)
         if (self.scroll_clear and self._scroll_offset > self.layout.width) or (
             not self.scroll_clear
-            and (self.layout.width - self._scroll_offset)
-            < (self._scroll_width - 2 * self.actual_padding)
+            and (self.layout.width - self._scroll_offset) < (self._scroll_width)
         ):
             self._is_scrolling = False
 
@@ -717,10 +701,13 @@ class _TextBox(_Widget):
         font: str | None = None,
         fontsize: int = 0,
         fontshadow: ColorType = "",
+        foreground: ColorType = "",
+        markup: bool | None = None,
     ):
         """
-        Change the font used by this widget. If font is None, the current
-        font is used.
+        Change the text layout properties and redraw the widget.
+        This method may also be used sync attributes from the current
+        widget with the text layout.
         """
         if font is not None:
             self.font = font
@@ -728,19 +715,29 @@ class _TextBox(_Widget):
             self.fontsize = fontsize
         if fontshadow != "":
             self.fontshadow = fontshadow
+        if foreground != "":
+            self.foreground = foreground
+        if markup is not None:
+            self.markup = markup
+        # Sync text layout properties
+        if self.layout:
+            self.layout.font_family = self.font
+            self.layout.font_size = self.fontsize
+            self.layout.font_shadow = self.fontshadow
+            self.layout.colour = self.foreground
+            self.layout.markup = self.markup
         self.bar.draw()
 
     @expose_command()
     def info(self):
         d = _Widget.info(self)
-        d["foreground"] = self.foreground
         d["text"] = self.formatted_text
         return d
 
     def update(self, text):
         """Update the widget text."""
         # Don't try to update text in dead layouts
-        # This is mainly required for ThreadPoolText based widgets as the
+        # This is mainly required for BackgroundPoll based widgets as the
         # polling function cannot be cancelled and so may be called after the widget
         # is finalised.
         if not self.can_draw():
@@ -756,7 +753,7 @@ class _TextBox(_Widget):
 
         # If our width hasn't changed, we just draw ourselves. Otherwise,
         # we draw the whole bar.
-        if self.layout.width == old_width:
+        if self.layout.width == old_width and (self.bar.horizontal or self.rotate):
             self.draw()
         else:
             self.bar.draw()
@@ -765,10 +762,10 @@ class _TextBox(_Widget):
 class InLoopPollText(_TextBox):
     """A common interface for polling some 'fast' information, munging it, and
     rendering the result in a text box. You probably want to use
-    ThreadPoolText instead.
+    BackgroundPoll instead.
 
     ('fast' here means that this runs /in/ the event loop, so don't block! If
-    you want to run something nontrivial, use ThreadedPollWidget.)"""
+    you want to run something nontrivial, use BackgroundPoll.)"""
 
     defaults = [
         (
@@ -793,14 +790,6 @@ class InLoopPollText(_TextBox):
             self.timeout_add(update_interval, self.timer_setup)
         # If update_interval is False, we won't re-call
 
-    def _configure(self, qtile, bar):
-        should_tick = self.configured
-        _TextBox._configure(self, qtile, bar)
-
-        # Update when we are being re-configured.
-        if should_tick:
-            self.tick()
-
     def button_press(self, x, y, button):
         self.tick()
         _TextBox.button_press(self, x, y, button)
@@ -812,15 +801,25 @@ class InLoopPollText(_TextBox):
         text = self.poll()
         self.update(text)
 
+    @expose_command()
+    def force_update(self):
+        """Immediately poll the widget. Existing timers are unaffected."""
+        self.tick()
 
-class ThreadPoolText(_TextBox):
+
+class BackgroundPoll(_TextBox):
     """A common interface for wrapping blocking events which when triggered
     will update a textbox.
 
-    The poll method is intended to wrap a blocking function which may take
-    quite a while to return anything.  It will be executed as a future and
-    should return updated text when completed.  It may also return None to
-    disable any further updates.
+    The poll/apoll methods are intended to wrap a blocking function which may
+    take quite a while to return anything. Either method should return the
+    string to update the widget text to. It may also return None to disable
+    any further updates.
+
+    If an `async def apoll()` is defined, that will be used to do the polling.
+
+    For widgets that have not been ported to asyncio and define a `def poll()`
+    method, their poll method will still be run in a thread as it is today.
 
     param: text - Initial text to display.
     """
@@ -833,52 +832,47 @@ class ThreadPoolText(_TextBox):
         ),
     ]  # type: list[tuple[str, Any, str]]
 
-    def __init__(self, text, **config):
+    def __init__(self, text="N/A", **config):
         super().__init__(text, **config)
-        self.add_defaults(ThreadPoolText.defaults)
+        self.add_defaults(BackgroundPoll.defaults)
 
     def timer_setup(self):
-        def on_done(future):
-            try:
-                result = future.result()
-            except Exception:
-                result = None
-                logger.exception("poll() raised exceptions, not rescheduling")
-
-            if result is not None:
-                try:
-                    self.update(result)
-
-                    if self.update_interval is not None:
-                        self.timeout_add(self.update_interval, self.timer_setup)
-
-                except Exception:
-                    logger.exception("Failed to reschedule timer for %s.", self.name)
-            else:
-                logger.warning("%s's poll() returned None, not rescheduling", self.name)
-
-        self.future = self.qtile.run_in_executor(self.poll)
-        self.future.add_done_callback(on_done)
+        create_task(self.do_tick())
 
     def poll(self):
-        pass
+        """An optional non-async-based method for polling. Will be run as an
+        async future."""
+
+    async def apoll(self):
+        """An optional async-based method for polling."""
+
+    async def do_tick(self, requeue=True):
+        if type(self).apoll != BackgroundPoll.apoll:
+            result = await self.apoll()
+        elif type(self).poll != BackgroundPoll.poll:
+            future = self.qtile.run_in_executor(self.poll)
+            result = await future
+        else:
+            raise Exception(f"widget {self.name} has neither apoll() nor poll() overridden?")
+        if result is not None:
+            try:
+                self.update(result)
+            except Exception:
+                logger.exception("Failed to reschedule timer for %s.", self.name)
+            if requeue and self.update_interval is not None:
+                await asyncio.sleep(self.update_interval)
+                create_task(self.do_tick())
+        else:
+            logger.warning("%s's poll() returned None, not rescheduling", self.name)
 
     @expose_command()
     def force_update(self):
         """Immediately poll the widget. Existing timers are unaffected."""
-        self.update(self.poll())
-
-
-# these two classes below look SUSPICIOUSLY similar
+        create_task(self.do_tick(requeue=False))
 
 
 class PaddingMixin(configurable.Configurable):
-    """Mixin that provides padding(_x|_y|)
-
-    To use it, subclass and add this to __init__:
-
-        self.add_defaults(base.PaddingMixin.defaults)
-    """
+    """Mixin that provides padding(_x|_y|)."""
 
     defaults = [
         ("padding", 3, "Padding inside the box"),
@@ -889,14 +883,21 @@ class PaddingMixin(configurable.Configurable):
     padding_x = configurable.ExtraFallback("padding_x", "padding")
     padding_y = configurable.ExtraFallback("padding_y", "padding")
 
+    @property
+    def padding_side(self):
+        if self.bar.horizontal:
+            return self.padding_x
+        return self.padding_y
+
+    @property
+    def padding_top(self):
+        if self.bar.horizontal:
+            return self.padding_y
+        return self.padding_x
+
 
 class MarginMixin(configurable.Configurable):
-    """Mixin that provides margin(_x|_y|)
-
-    To use it, subclass and add this to __init__:
-
-        self.add_defaults(base.MarginMixin.defaults)
-    """
+    """Mixin that provides margin(_x|_y|)."""
 
     defaults = [
         ("margin", 3, "Margin inside the box"),
@@ -907,6 +908,18 @@ class MarginMixin(configurable.Configurable):
     margin_x = configurable.ExtraFallback("margin_x", "margin")
     margin_y = configurable.ExtraFallback("margin_y", "margin")
 
+    @property
+    def margin_side(self):
+        if self.bar.horizontal:
+            return self.margin_x
+        return self.margin_y
+
+    @property
+    def margin_top(self):
+        if self.bar.horizontal:
+            return self.margin_y
+        return self.margin_x
+
 
 class Mirror(_Widget):
     """
@@ -965,7 +978,7 @@ class Mirror(_Widget):
             return
         self.drawer.clear_rect()
         self.reflects.drawer.paint_to(self.drawer)
-        self.drawer.draw(offsetx=self.offset, offsety=self.offsety, width=self.width)
+        self.draw_at_default_position()
 
     def button_press(self, x, y, button):
         self.reflects.button_press(x, y, button)
